#!/usr/bin/perl


## Recuerda cambiar la lnea
##                  #!/usr/bin/perl
## de acuerdo al path de tu Perl

#############
##
## PROGRAMA PARA IPN DE PAYPAL
## AUTOR: Uriel Lizama (uriel@baboonsoftware.com)
## URL: http://perlenespanol.baboonsoftware.com/
## VERSIN: 0.2
##
##
## No hay garanta sobre el uso de este script. sese bajo su propio riesgo.
##
## Para mayor informacin entrar a:
## http://perlenespanol.baboonsoftware.com/tutoriales/
##
##############


use strict;

# Cargamos los Mdulos
use CGI::Carp;
use CGI qw(:standard escapeHTML);
use LWP::UserAgent;


# CGI.pm versioning issue
$CGI::USE_PARAM_SEMICOLONS = 0;


# -------------------------------------------------------------------
#   CONFIGURA LAS SIGUIENTE VARIABLES

# URL a PayPal
my $PAYPAL_URL = 'http://www.paypal.com/cgi-bin/webscr';

#Archivo .htpasswd
my $HTPASSWD = '/home/ruta/a/tu/.htpasswd';

# Direccin de nuestro sendmail
my $SENDMAIL_PATH = '/usr/sbin/sendmail';

# Direccin donde queremos las notificaciones de error
my $ADMIN_EMAIL = 'notificaciones@misitio.com';

# Nuestro Email de la cuenta de PayPal
my @PAYMENT_EMAILS = ('pagos@misitio.com');
# -------------------------------------------------------------------


#Iniciamos el proceso
main();


sub main {

        # Verificamos el IPN
        if (verificar_ipn()) {
           # Ha sido verificado vamos a procesarlo
           procesar_ipn();
        }

        # IPN was successfully processed
        respuesta(1);
}


sub verificar_ipn {

    # Nuevo UserAgent
    my $ua = new LWP::UserAgent;

    # Creamos el request
    my $req = new HTTP::Request("POST", $PAYPAL_URL);
       $req->content_type("application/x-www-form-urlencoded");
       $req->content(query_string() . "&cmd=_notify-validate");

    # Recibimos la respuesta
    my $resp = $ua->request($req);

    # Checamos si fue exitosa la respuesta
    if (($resp->is_success) && ($resp->content eq "VERIFIED")) {
        return 1;
    } else {
        # Intentamos identificar el error
        if (($resp->is_success) && ($resp->content eq "INVALID")) {
            procesar_error("La notificacin recibida no fue de PayPal - El mensaje fue ignorado ", "verificacin de IPN", 0, 0);
        } else {
            procesar_error("La notificacin no pudo ser procesada por un problema con PayPal.", "verificacin de IPN", 1, 0);
        }

        return undef;
    }

}


sub procesar_ipn {

        # Nueva membresa
        if ((param("txn_type") eq "subscr_signup") && (verificar_cuenta())) {
                # agregamos el usuario
                activar_usuario(param("username"),param("password"));
        } elsif (param("txn_type") eq "subscr_eot") {
                # eliminamos al usuario
                desactivar_usuario(param("username"));
        } else {
                # ignoramos el mensaje
        }

}


sub verificar_cuenta {

        # validamos que el pago se haya realizado a nuestra cuenta
        my $valid = undef;

        for my $pay_mail (@PAYMENT_EMAILS) {
                if (param("receiver_email") eq $pay_mail) {
                        $valid = 1;
                        last;
                }
        }

        if (!$valid){
               procesar_error("Se recibi un IPN de una direccin de email que no es la tuya - El mensaje fue ignorado.",
                              "validacin de email ", 0, 0);
                return undef;
        }

        return 1;
}


sub procesar_error {
        # sends notification that an error has occured
        my $err_str = shift;
        my $action = shift;
        my $kill = shift;
        my $req_action = shift;

        my $message = "El siguiente error fue encontrado al intentar hacer la $action: \n\t$err_str\n\n\n";
        $message .= "Informacin del Usuario\n";
        $message .= "\tUsername: " . param("username") . "\n";
        $message .= "\tEmail: " . param("payer_email") . "\n";
        $message .= "\tNmero de Suscripcin: " . param("subscr_id") . "\n";
        $message .= "\tTipo de Transaccin: " . param("txn_type") . "\n";

        my $subject = "Error de Suscripcin";
        if ($req_action) {
                $subject .= " - Necesita una accin";
        } else {
                $subject .= " - No se necesita realizar nada";
        }

        # if an email is not specified write to error_log only
        if (($ADMIN_EMAIL) && ($SENDMAIL_PATH)) {

            if (!open(SENDMAIL, "|$SENDMAIL_PATH -t")) {
                    carp "Unable to open sendmail pipe.";
            }

            print SENDMAIL "To: $ADMIN_EMAIL\n";
            print SENDMAIL "From: $ADMIN_EMAIL\n";
            print SENDMAIL "Subject: $subject\n";
            print SENDMAIL "Content-type: text/plain\n\n";
            print SENDMAIL "$message";

            if (!close(SENDMAIL)) {
                carp "Unable to close sendmail pipe.";
            }

        }

        # put it into the error log
        if ($kill) {
                # IPN will retry
                respuesta(0);
                croak $message;
        } else {
                carp $message;
        }
}



sub activar_usuario{

my $user = shift;
my $pass = shift;


#Verificamos que el usuario no exista ya
my $existe = undef;

open my $in, '<', $HTPASSWD or procesar_error("No se pudo abrir archivo .htpasswd - $!", "abrir htpasswd", 0, 1);
    while(my $line = <$in>){
        my ($auser,$apass) = split(/\:/, $line);
            if($auser eq $user){
                $existe = 1;
                last;
            }
     }
close $in;


if($existe){
    # El usuario si existe, mandamos error
    procesar_error("El usuario $user ya existe - No se ha hecho ninguna accin", "crear usuario", 0, 1);
    return 1;
}else{
    # Introducimos el usuario a nuestro .htpasswd
    open my $out, '>>', $HTPASSWD or procesar_error("No se pudo abrir archivo .htpasswd - $!", "abrir htpasswd", 0, 1);
        print {$out} $user . ":" . $pass;
        print {$out} "\n";
    close $out;

    return 1;
}


}


sub desactivar_usuario{

my $user = shift;


open my $in, '<', $HTPASSWD or procesar_error("No se pudo abrir archivo .htpasswd - $!", "abrir htpasswd", 0, 1);
my @usuarios = <$in>;
close $in;

    # Introducimos el usuario a nuestro .htpasswd
    open my $out, '>', $HTPASSWD or procesar_error("No se pudo abrir archivo .htpasswd - $!", "abrir htpasswd", 0, 1);
        for my $linea_usuario(@usuarios){
            chomp($linea_usuario);

            my($auser,$apass) = split(/\:/, $linea_usuario);

            if($auser ne $user){
                print {$out} $linea_usuario, "\n";
            }
        }

    close $out;


return 1;

}



sub respuesta {
        # handle the http reponse
        my $is_success = shift;

        if ($is_success) {
                print header(-status=>('204 No Content'));
        }
        else {
                print header(-status=>('500 Internal Server Error'));
        }
}
